﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Kokomo.Mirror.Ia32
{
	public class IndirectOperand : Operand
	{
		public IndirectOperand(
			OperandSize size,
			SimpleOperand baseOperand,
			long displacement = 0,
			Register segmentOverride = null
			)
			: base(size)
		{
			if (baseOperand == null) throw new ArgumentNullException("baseOperand");

			this.BaseOperand = baseOperand;
			this.Displacement = displacement;
			this.SegmentOverride = segmentOverride;
		}

		public SimpleOperand BaseOperand { get; private set; }
		public long Displacement { get; private set; }
		public Register SegmentOverride { get; private set; }

		public override bool Equals(object obj)
		{
			IndirectOperand other = obj as IndirectOperand;
			if (other == null) return false;

			bool equal =
				(object.Equals(this.BaseOperand, other.BaseOperand))
				&& (this.Displacement == other.Displacement)
				&& (object.Equals(this.SegmentOverride, other.SegmentOverride))
				;
			return equal;
		}
		public override int GetHashCode()
		{
			int hash = 0;
			if (this.BaseOperand != null) hash ^= this.BaseOperand.GetHashCode();
			hash ^= this.Displacement.GetHashCode();
			if (this.SegmentOverride != null) hash ^= this.SegmentOverride.GetHashCode();

			return hash;
		}

		public override string ToString()
		{
			StringBuilder builder = new StringBuilder();
			builder.Append("[");
			if (this.SegmentOverride != null)
			{
				builder.Append(this.SegmentOverride);
				builder.Append(":");
			}
			builder.Append(BaseOperand);
			if (this.Displacement > 0)
			{
				builder.Append("+");
				builder.AppendFormat("0{0:X}h", this.Displacement);
			}
			else if (this.Displacement < 0)
			{
				builder.Append("-");
				builder.AppendFormat("0{0:X}h", -this.Displacement);
			}
			builder.Append("]");

			return builder.ToString();
		}
	}
}
